# begin_generated_IBM_copyright_prolog
#
# Licensed Materials - Property of IBM
# 5724-N72 5655-R41
# (c) Copyright IBM Corp. 2006, 2007 All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# end_generated_IBM_copyright_prolog

#===============================================================================
# USAGE:
#    wsadmin  -lang jython -f installApprovalProcessApps.py
#               standalone|cluster servername|clustername cellname nodename earlocation 
#               
#    (enter the above all on one command line)   
#===============================================================================
# arg0 - flag decides standalone|cluster
# arg1 - server name or cluster name based on the flag
# arg2 - cell name ,optional
# arg3 - node name,optional
# arg4 - ear location
#===============================================================================

import sys
import os
import glob
import time
#---------------------------------------------------------
   # Set Topology
#---------------------------------------------------------
 
def setTopology(flag):
   global serverTypeID, installType
   flag = flag.lower()
   if flag == "cluster":
      serverTypeID="/ServerCluster:"
      installType="-cluster"
      
   elif flag == "standalone":
      serverTypeID="/Server:"
      installType="-server"
      
   else:
      print " Wrong flag used! Please specify it only with 'cluster' or 'standalone'!"
	  
#---------------------------------------------------------
   #  Check for node
#---------------------------------------------------------	  
def checkNode(nodeName):
      node = AdminConfig.getid("/Node:" + nodeName + "/")
      if len(node) == 0:
        print "could not find node " + nodeName
        sys.exit(1)

#---------------------------------------------------------
   # Check for cluster
#---------------------------------------------------------		
def checkCluster(clusterName):
   #---------------------------------------------------------
   # Get the config id for the Cluster
   #---------------------------------------------------------
   #print serverTypeID
   cluster = AdminConfig.getid(serverTypeID + clusterName + "/")
   if len(cluster) == 0:
       print " could not find cluster/server " + clusterName
       sys.exit(1)


#---------------------------------------------------------
   #  Check for Usage
#---------------------------------------------------------	  
def usageHelp():
   print "Usage: this script requires the following parameters: "
   print "           1) Flag standalone or cluster, "
   print "           2) cluster or server name (depends on the 1. parameter), "
   print "           3) [cellName] required only if the flag is standalone"
   print "           4) [nodename] required only if the flag is standalone"      
   print "           5) EAR location"
   print ""
   print "e.g.:  wsadmin -lang jython -f installApprovalProcessApps.py" + " cluster cluster1 C:/tmp "
   sys.exit(1)	   
def install(flag, clusterName, cellName, nodeName, earLocation):
		#---------------------------------------------------------
		# Main Method for installation of EAR
		# This Method will install EAR into Cluster or Standalone. 
		# depending on FLAG
		#---------------------------------------------------------
		print " Start of script: " 
		print " WPS Approval Process applications will be installed in server mode : " + flag
		
		  
		# set up globals
		#--------------------------------------------------------------
		global AdminConfig
		global AdminControl
		global AdminApp
		global AdminTask
		global serverTypeI
		global installType
		appNames = ['WSRRGovSRInterfaceApp','WSRRGovApprovalProxyProcessApp','WSRRGovApprovalApp']
		lineSeparator = java.lang.System.getProperty('line.separator')
		setTopology(flag)
		if flag == "standalone":
		   checkNode(nodeName)
		checkCluster(clusterName)

		clusterOpt = [installType, clusterName]

		appOptions = []
		appOptions.extend(clusterOpt)

		if flag == "standalone":
		  nodeOpt = ["-node", nodeName]
		  appOptions.extend(nodeOpt)
	  	
		earFound=0
		for earName in glob.glob(os.path.join(earLocation, '*.ear')):
		   	earFound=1
		if earFound==0:
			print "couldn't found ears on the speified earLocation"
			sys.exit(1)
		# Check if application exists before proceeding
		for appName in appNames:
			app = AdminConfig.getid('/Deployment:' + appName + '/')
			if (len(app) > 0):
				print "The application "+appName +" exists, proceed to uninstall current application." + "\n"
				AdminApp.uninstall(appName)
	   #---------------------------------------------------------
		# Install the EARS from the location
	   #---------------------------------------------------------
		for earName in glob.glob(os.path.join(earLocation, '*.ear')):
		   try:
				AdminApp.install(earName, appOptions)
				
		   except:
				exc_info = sys.exc_info()
				print " Error during Application installation:"+exc_info
		#---------------------------------------------------------
		# save  the configuration
		#---------------------------------------------------------
		AdminConfig.save()  
		
		#---------------------------------------------------------
		# start  the apps
		# To start or stop the application running in a cluster, the user must run wsadmin scriping start or stop commands for all the cluster members separately.
		#---------------------------------------------------------
		if flag == "cluster":
			clusterId =AdminConfig.getid('/ServerCluster:'+clusterName+'/')
			clusterMembers =AdminConfig.list("ClusterMember", clusterId).split(lineSeparator)
			#print "Starting the Applications in Cluster"
			for m_id in clusterMembers:
				#print"m_id "+ m_id
				currentServer =AdminConfig.showAttribute(m_id, 'memberName')
				currentNodeName =AdminConfig.showAttribute(m_id ,'nodeName')
				currentCellName=AdminControl.getCell()
				appManager = AdminControl.queryNames('cell='+currentCellName+',node='+currentNodeName+',type=ApplicationManager,process='+currentServer+',*')
				for eachApp in appNames:
					#myRunningApp =AdminControl.queryNames('type=Application,name='+eachApp+',*')
					myRunningApp =AdminControl.queryNames('cell='+currentCellName+',node='+currentNodeName+',type=Application,process='+currentServer+',name='+eachApp+',*')
					isDistributed=AdminApp.isAppReady(eachApp)
					# "App eachApp is not started"
					if (len(myRunningApp) == 0):
						if(isDistributed =="false"):
							print "EAR is still being distributed to cluster members.Waiting for 1 minute..."
							time.sleep(60)
							isDistributed=AdminApp.isAppReady(eachApp)
							if(isDistributed =="true"):
								print "Starting the App "+eachApp
								AdminControl.invoke(appManager, 'startApplication', eachApp)
							else:
								print "EARS not distributed to cluster members.Exiting...Use AdminConsole to start the Apps"
								sys.exit(1)
						else:
							print "Starting the App "+eachApp
							AdminControl.invoke(appManager, 'startApplication', eachApp)
		else:
			#print "Starting the Applications in Server"
			appManager = AdminControl.queryNames('cell='+cellName+',node='+nodeName+',type=ApplicationManager,process='+clusterName+',*')
			for eachApp in appNames:
				myRunningApp =AdminControl.queryNames('type=Application,name='+eachApp+',*')
				if (len(myRunningApp) == 0):
					# "App eachApp is not running"
					AdminControl.invoke(appManager, 'startApplication', eachApp)
					
		print "Approval Process Applications installed and started successfully."	
				
#------------------------------------------------------------------
# identify cluster or standalone and call appropriate methods
#------------------------------------------------------------------
if ((len(sys.argv) < 3)):
	usageHelp()		
	sys.exit(1)		
elif (sys.argv[0]=="cluster"):
	if ((len(sys.argv) != 3)):
		usageHelp()
	elif ((os.path.isdir(sys.argv[2])) == 0):
		print "EAR location is not a valid path/directory"
		sys.exit(1)
	else:
		install(sys.argv[0], sys.argv[1], " ", " ", sys.argv[2])
elif (sys.argv[0]=="standalone"):
	if ((len(sys.argv) != 5)):
		usageHelp()		
	elif ((os.path.isdir(sys.argv[4])) == 0):
		 print "EAR location is not a valid path/directory"
		 sys.exit(1)
	else:
		install(sys.argv[0], sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
	 
